@@ -1,4 +1,7 @@
 import json
+import warnings
+from cqlengine import SizeTieredCompactionStrategy, LeveledCompactionStrategy
+from cqlengine.named import NamedTable
 
 from cqlengine.connection import connection_manager, execute
 from cqlengine.exceptions import CQLEngineException
@@ -10,6 +13,10 @@ Field = namedtuple('Field', ['name', 'type'])
 logger = logging.getLogger(__name__)
 
 
+# system keyspaces
+schema_columnfamilies = NamedTable('system', 'schema_columnfamilies')
+
+
 def create_keyspace(name, strategy_class='SimpleStrategy', replication_factor=3, durable_writes=True, **replication_values):
     """
     creates a keyspace
@@ -47,8 +54,11 @@ def delete_keyspace(name):
         if name in [r[0] for r in keyspaces]:
             execute("DROP KEYSPACE {}".format(name))
 
-
 def create_table(model, create_missing_keyspace=True):
+    warnings.warn("create_table has been deprecated in favor of sync_table and will be removed in a future release", DeprecationWarning)
+    sync_table(model, create_missing_keyspace)
+
+def sync_table(model, create_missing_keyspace=True):
 
     if model.__abstract__:
         raise CQLEngineException("cannot create table from abstract model")
@@ -70,36 +80,8 @@ def create_table(model, create_missing_keyspace=True):
     tables = [x[0] for x in tables.results]
 
     #check for an existing column family
-    #TODO: check system tables instead of using cql thrifteries
     if raw_cf_name not in tables:
-        qs = ['CREATE TABLE {}'.format(cf_name)]
-
-        #add column types
-        pkeys = [] # primary keys
-        ckeys = [] # clustering keys
-        qtypes = [] # field types
-        def add_column(col):
-            s = col.get_column_def()
-            if col.primary_key:
-                keys = (pkeys if col.partition_key else ckeys)
-                keys.append('"{}"'.format(col.db_field_name))
-            qtypes.append(s)
-        for name, col in model._columns.items():
-            add_column(col)
-
-        qtypes.append('PRIMARY KEY (({}){})'.format(', '.join(pkeys), ckeys and ', ' + ', '.join(ckeys) or ''))
-
-        qs += ['({})'.format(', '.join(qtypes))]
-
-        with_qs = ['read_repair_chance = {}'.format(model.__read_repair_chance__)]
-
-        _order = ['"{}" {}'.format(c.db_field_name, c.clustering_order or 'ASC') for c in model._clustering_keys.values()]
-        if _order:
-            with_qs.append('clustering order by ({})'.format(', '.join(_order)))
-
-        # add read_repair_chance
-        qs += ['WITH {}'.format(' AND '.join(with_qs))]
-        qs = ' '.join(qs)
+        qs = get_create_table(model)
 
         try:
             execute(qs)
@@ -121,6 +103,8 @@ def create_table(model, create_missing_keyspace=True):
             logger.debug(query)
             execute(query)
 
+        update_compaction(model)
+
 
     #get existing index names, skip ones that already exist
     with connection_manager() as con:
@@ -147,6 +131,89 @@ def create_table(model, create_missing_keyspace=True):
                 # index already exists
                 pass
 
+def get_create_table(model):
+    cf_name = model.column_family_name()
+    qs = ['CREATE TABLE {}'.format(cf_name)]
+
+    #add column types
+    pkeys = [] # primary keys
+    ckeys = [] # clustering keys
+    qtypes = [] # field types
+    def add_column(col):
+        s = col.get_column_def()
+        if col.primary_key:
+            keys = (pkeys if col.partition_key else ckeys)
+            keys.append('"{}"'.format(col.db_field_name))
+        qtypes.append(s)
+    for name, col in model._columns.items():
+        add_column(col)
+
+    qtypes.append('PRIMARY KEY (({}){})'.format(', '.join(pkeys), ckeys and ', ' + ', '.join(ckeys) or ''))
+
+    qs += ['({})'.format(', '.join(qtypes))]
+
+    with_qs = ['read_repair_chance = {}'.format(model.__read_repair_chance__)]
+
+    _order = ['"{}" {}'.format(c.db_field_name, c.clustering_order or 'ASC') for c in model._clustering_keys.values()]
+
+    if _order:
+        with_qs.append('clustering order by ({})'.format(', '.join(_order)))
+
+    compaction_options = get_compaction_options(model)
+
+    if compaction_options:
+        compaction_options = json.dumps(compaction_options).replace('"', "'")
+        with_qs.append("compaction = {}".format(compaction_options))
+
+    # add read_repair_chance
+    qs += ['WITH {}'.format(' AND '.join(with_qs))]
+
+
+    qs = ' '.join(qs)
+    return qs
+
+
+def get_compaction_options(model):
+    """
+    Generates dictionary (later converted to a string) for creating and altering
+    tables with compaction strategy
+
+    :param model:
+    :return:
+    """
+    if not model.__compaction__:
+        return {}
+
+    result = {'class':model.__compaction__}
+
+    def setter(key, limited_to_strategy = None):
+        """
+        sets key in result, checking if the key is limited to either SizeTiered or Leveled
+        :param key: one of the compaction options, like "bucket_high"
+        :param limited_to_strategy: SizeTieredCompactionStrategy, LeveledCompactionStrategy
+        :return:
+        """
+        mkey = "__compaction_{}__".format(key)
+        tmp = getattr(model, mkey)
+        if tmp and limited_to_strategy and limited_to_strategy != model.__compaction__:
+            raise CQLEngineException("{} is limited to {}".format(key, limited_to_strategy))
+
+        if tmp:
+            result[key] = tmp
+
+    setter('tombstone_compaction_interval')
+
+    setter('bucket_high', SizeTieredCompactionStrategy)
+    setter('bucket_low', SizeTieredCompactionStrategy)
+    setter('max_threshold', SizeTieredCompactionStrategy)
+    setter('min_threshold', SizeTieredCompactionStrategy)
+    setter('min_sstable_size', SizeTieredCompactionStrategy)
+
+    setter("sstable_size_in_mb", LeveledCompactionStrategy)
+
+    return result
+
+
 def get_fields(model):
     # returns all fields that aren't part of the PK
     ks_name = model._get_keyspace()
@@ -162,7 +229,48 @@ def get_fields(model):
     return [Field(x[0], x[1]) for x in tmp.results]
     # convert to Field named tuples
 
+
+def get_table_settings(model):
+    return schema_columnfamilies.get(keyspace_name=model._get_keyspace(),
+                                     columnfamily_name=model.column_family_name(include_keyspace=False))
+
+
+def update_compaction(model):
+    logger.debug("Checking %s for compaction differences", model)
+    row = get_table_settings(model)
+    # check compaction_strategy_class
+    if not model.__compaction__:
+        return
+
+    do_update = not row['compaction_strategy_class'].endswith(model.__compaction__)
+
+    existing_options = row['compaction_strategy_options']
+    existing_options = json.loads(existing_options)
+
+    desired_options = get_compaction_options(model)
+    desired_options.pop('class', None)
+
+    for k,v in desired_options.items():
+        val = existing_options.pop(k, None)
+        if val != v:
+            do_update = True
+
+    # check compaction_strategy_options
+    if do_update:
+        options = get_compaction_options(model)
+        # jsonify
+        options = json.dumps(options).replace('"', "'")
+        cf_name = model.column_family_name()
+        query = "ALTER TABLE {} with compaction = {}".format(cf_name, options)
+        logger.debug(query)
+        execute(query)
+
+
 def delete_table(model):
+    warnings.warn("delete_table has been deprecated in favor of drop_table()", DeprecationWarning)
+    return drop_table(model)
+
+def drop_table(model):
 
     # don't try to delete non existant tables
     ks_name = model._get_keyspace()
@@ -178,3 +286,4 @@ def delete_table(model):
     cf_name = model.column_family_name()
     execute('drop table {};'.format(cf_name))
 
+
