@@ -7,10 +7,12 @@ from cqlengine.query import ModelQuerySet, DMLQuery, AbstractQueryableColumn
 from cqlengine.query import DoesNotExist as _DoesNotExist
 from cqlengine.query import MultipleObjectsReturned as _MultipleObjectsReturned
 
+
 class ModelDefinitionException(ModelException): pass
 
 DEFAULT_KEYSPACE = 'cqlengine'
 
+
 class hybrid_classmethod(object):
     """
     Allows a method to behave as both a class method and
@@ -44,7 +46,7 @@ class QuerySetDescriptor(object):
         """ :rtype: ModelQuerySet """
         if model.__abstract__:
             raise CQLEngineException('cannot execute queries against abstract models')
-        return ModelQuerySet(model)
+        return model.__queryset__(model)
 
     def __call__(self, *args, **kwargs):
         """
@@ -70,6 +72,7 @@ class ColumnQueryEvaluator(AbstractQueryableColumn):
     def _get_column(self):
         return self.column
 
+
 class ColumnDescriptor(object):
     """
     Handles the reading and writing of column values to and from
@@ -127,6 +130,7 @@ class BaseModel(object):
     """
 
     class DoesNotExist(_DoesNotExist): pass
+
     class MultipleObjectsReturned(_MultipleObjectsReturned): pass
 
     objects = QuerySetDescriptor()
@@ -138,6 +142,27 @@ class BaseModel(object):
     #the keyspace for this model
     __keyspace__ = None
 
+
+    # compaction options
+    __compaction__ = None
+    __compaction_tombstone_compaction_interval__ = None
+    __compaction_tombstone_threshold = None
+
+    # compaction - size tiered options
+    __compaction_bucket_high__ = None
+    __compaction_bucket_low__ = None
+    __compaction_max_threshold__ = None
+    __compaction_min_threshold__ = None
+    __compaction_min_sstable_size__ = None
+
+    # compaction - leveled options
+    __compaction_sstable_size_in_mb__ = None
+
+    # end compaction
+    # the queryset class used for this class
+    __queryset__ = ModelQuerySet
+    __dmlquery__ = DMLQuery
+
     __read_repair_chance__ = 0.1
 
     def __init__(self, **values):
@@ -259,7 +284,7 @@ class BaseModel(object):
     def save(self):
         is_new = self.pk is None
         self.validate()
-        DMLQuery(self.__class__, self, batch=self._batch).save()
+        self.__dmlquery__(self.__class__, self, batch=self._batch).save()
 
         #reset the value managers
         for v in self._values.values():
@@ -270,7 +295,7 @@ class BaseModel(object):
 
     def delete(self):
         """ Deletes this instance """
-        DMLQuery(self.__class__, self, batch=self._batch).delete()
+        self.__dmlquery__(self.__class__, self, batch=self._batch).delete()
 
     @classmethod
     def _class_batch(cls, batch):
@@ -316,19 +341,26 @@ class ModelMetaClass(type):
 
         column_definitions = inherited_columns.items() + column_definitions
 
-        #columns defined on model, excludes automatically
-        #defined columns
         defined_columns = OrderedDict(column_definitions)
 
         #prepend primary key if one hasn't been defined
         if not is_abstract and not any([v.primary_key for k,v in column_definitions]):
             raise ModelDefinitionException("At least 1 primary key is required.")
 
+        counter_columns = [c for c in defined_columns.values() if isinstance(c, columns.Counter)]
+        data_columns = [c for c in defined_columns.values() if not c.primary_key and not isinstance(c, columns.Counter)]
+        if counter_columns and data_columns:
+            raise ModelDefinitionException('counter models may not have data columns')
+
         has_partition_keys = any(v.partition_key for (k, v) in column_definitions)
 
         #TODO: check that the defined columns don't conflict with any of the Model API's existing attributes/methods
         #transform column definitions
         for k,v in column_definitions:
+            # counter column primary keys are not allowed
+            if (v.primary_key or v.partition_key) and isinstance(v, (columns.Counter, columns.BaseContainerColumn)):
+                raise ModelDefinitionException('counter columns and container columns cannot be used as primary keys')
+
             # this will mark the first primary key column as a partition
             # key, if one hasn't been set already
             if not has_partition_keys and v.primary_key:
@@ -382,6 +414,7 @@ class ModelMetaClass(type):
 
         attrs['_partition_keys'] = partition_keys
         attrs['_clustering_keys'] = clustering_keys
+        attrs['_has_counter'] = len(counter_columns) > 0
 
         #setup class exceptions
         DoesNotExistBase = None
